<?php
defined( 'ABSPATH' ) or die('This page may not be accessed directly.');

class WonderPushUtils {
  public static function is_wonderpush_installed() {
    $settings = WonderPushSettings::getSettings();
    $access_token = $settings->getAccessToken();
    if (!$access_token || $settings->getDisableSnippet()) return false;
    try {
      $app = self::application_from_access_token($access_token);
    } catch (Exception $e) {
      return false;
    }
    if (!$app) return false;
    if (!$app->getWebKey()) return false;
    return true;
  }

  public static function can_modify_settings() {
    return WonderPushUtils::is_admin_user();
  }

  public static function can_send_notifications() {
    return current_user_can('publish_posts') || current_user_can('edit_published_posts');
  }

  public static function is_admin_user() {
    return current_user_can('delete_users');
  }

  /* If >= PHP 5.4, ENT_HTML401 | ENT_QUOTES will correctly decode most entities including both double and single quotes.
   In PHP 5.3, ENT_HTML401 does not exist, so we have to use `str_replace("&apos;","'", $value)` before feeding it to html_entity_decode(). */
  public static function decode_entities($string) {
    if (!$string) return '';
    $HTML_ENTITY_DECODE_FLAGS = ENT_QUOTES;
    if (defined('ENT_HTML401')) {
      // @codingStandardsIgnoreLine
      $HTML_ENTITY_DECODE_FLAGS = ENT_HTML401 | $HTML_ENTITY_DECODE_FLAGS;
    }
    return html_entity_decode(str_replace("&apos;", "'", $string), $HTML_ENTITY_DECODE_FLAGS, 'UTF-8');
  }

  /**
   * Returns the first application associated with the provided access token,
   * or false if the access token is not valid.
   * This method uses the WordPress transient API to cache the application and avoid network calls if possible.
   * Throws exception is validity could not be determined (network error for instance).
   * @throws Exception
   * @param string $access_token
   * @param int $expiration How long to cache the result
   * @return false|\WonderPush\Obj\Application
   */
  public static function application_from_access_token($access_token, $expiration = 3600) {
    // Cached value ?
    $cache_key = "WonderPush:Application:" . $access_token;
    $cached = get_transient($cache_key);
    if ($cached) {
      return $cached;
    }

    // Check access token with the API
    $wp = new \WonderPush\WonderPush($access_token);
    try {
      $applications = $wp->applications()->all();
      if ($applications && $applications->getCount() > 0) {
        $data = $applications->getData();
        $app = $data[0];
        set_transient($cache_key, $app, $expiration);
        return $app;
      }
      // Access token not associated with any app
      delete_transient($cache_key);
      return false;
    } catch (Exception $e) {
      delete_transient($cache_key);
      if ($e instanceof \WonderPush\Errors\Server
        && ($e->getResponse()->getStatusCode() == 403 || $e->getCode() === 11003)) {
        // Invalid access token
        return false;
      }
      throw $e;
    }
  }

  /**
   * Puts the given properties on the current installation by calling WonderPush Management API
   * @param string $access_token
   * @param array|object $properties
   * @return boolean Success
   * @throws \WonderPush\Errors\Base
   */
  public static function put_current_installation_properties($access_token, $properties) {
    if (!$access_token) {
      return false;
    }
    $wp = new \WonderPush\WonderPush($access_token);
    $installation_id = self::get_installation_id();
    if (!$installation_id) {
      return false;
    }
    $params = new \WonderPush\Params\PatchInstallationParams($installation_id, '');
    $params->setProperties($properties);
    $result = $wp->installations()->patch($params);
    return $result->isSuccess();
  }

  /**
   * Returns the WonderPush installation ID for the current request
   * by looking at the cookie named `WonderPushPublic::INSTALLATION_ID_COOKIE_NAME`
   * @return string|null
   */
  public static function get_installation_id() {
    if (!array_key_exists(WonderPushPublic::INSTALLATION_ID_COOKIE_NAME, $_COOKIE)) {
      return null;
    }
    $installation_id = $_COOKIE[WonderPushPublic::INSTALLATION_ID_COOKIE_NAME];
    return $installation_id ? $installation_id : null;
  }

  /**
   * @return WooCommerce | null
   */
  public static function get_woocommerce() {
    if (!function_exists('WC')) return null;
    return WC();
  }

  /**
   * Converts a DateTime object to a string representing its date.
   * @param DateTime $date_time
   * @return string
   */
  public static function datetime_to_date_string($date_time) {
    return $date_time->format('Ymd');
  }

  /**
   * Converts a date string obtained with `datetime_to_date_string` to a DateTime object.
   * @param string $date_string A date_string in the form YYYYMMDD (8 digits exactly)
   * @return DateTime|null
   */
  public static function date_string_to_datetime($date_string) {
    if (!self::is_valid_date_string($date_string)) return null;
    $year = substr($date_string, 0, 4);
    $month = substr($date_string, 4, 2);
    $day = substr($date_string, 6, 2);
    $result = new DateTime();
    $result->setDate($year, $month, $day);
    $result->setTime(0, 0, 0, 0);
    return $result;
  }

  /**
   * Returns true when provided date string is exactly 8 digits.
   * @param $date_string
   * @return bool
   */
  public static function is_valid_date_string($date_string) {
    return preg_match('/\d{8}/', $date_string) ? true : false;
  }

  /**
   * Returns true when provided time string is an integer between 0 and 23 inclusive.
   * @param $time_string
   * @return bool
   */
  public static function is_valid_time_string($time_string) {
    return preg_match('/^[0-2]?[0-9]$/', $time_string) && (int)$time_string < 24 ? true : false;
  }

  public static function utm_parameters() {
      return array('utm_source', 'utm_medium', 'utm_campaign', 'utm_term', 'utm_content');
  }

  public static function inject_query_string_params($url, $params) {
    $parsed_url = parse_url($url);
    if ($parsed_url === false) return $url;
    return http_build_url($parsed_url, array('query' => http_build_query($params)), HTTP_URL_JOIN_QUERY);
  }
}
