<?php

class WonderPushWooCommerce {
  const CART_REMINDER_STRATEGY_LATEST = 'latest';
  const CART_REMINDER_STRATEGY_MOST_EXPENSIVE = 'most-expensive';
  const CART_REMINDER_STRATEGY_LEAST_EXPENSIVE = 'least-expensive';
  public static function cart_reminder_strategies() {
    return array(
      self::CART_REMINDER_STRATEGY_LATEST,
      self::CART_REMINDER_STRATEGY_MOST_EXPENSIVE,
      self::CART_REMINDER_STRATEGY_LEAST_EXPENSIVE,
    );
  }

  const CART_REMINDER_DESTINATION_CART = 'cart';
  const CART_REMINDER_DESTINATION_CHECKOUT = 'checkout';
  const CART_REMINDER_DESTINATION_HOMEPAGE = 'homepage';
  public static function cart_reminder_destinations() {
    return array(
      self::CART_REMINDER_DESTINATION_CART,
      self::CART_REMINDER_DESTINATION_CHECKOUT,
      self::CART_REMINDER_DESTINATION_HOMEPAGE,
    );
  }

  /** @var WooCommerce */
  static $woocommerce;
  static function init() {
    self::$woocommerce = WonderPushUtils::get_woocommerce();
    if (!self::$woocommerce) return;
    $cart_change_hooks = array(
      'woocommerce_add_to_cart',
      'woocommerce_cart_item_removed',
      'woocommerce_cart_item_restored',
      'woocommerce_cart_item_set_quantity',
      'woocommerce_cart_emptied',
      'woocommerce_thankyou',
    );
    foreach ($cart_change_hooks as $hook) {
      add_action($hook, array(__CLASS__, 'cart_did_change'));
    }
  }
  public static function cart_did_change() {
    if (!self::$woocommerce) return;
    $access_token = WonderPushSettings::getSettings()->getAccessToken();

    // We need an access token
    if (!$access_token) {
      return;
    }

    // Is the cart reminder enabled?
    if (!WonderPushSettings::getSettings()->getEnableCartReminder()) {
      return;
    }

    // Choose a cart reminder strategy
    $strategy = WonderPushSettings::getSettings()->getCartReminderStrategy();
    if (!$strategy) $strategy = self::CART_REMINDER_STRATEGY_LATEST;

    // Select a cart item
    $selected_cart_item = null;
    $selected_cart_item_total = 0;
    $cart = self::$woocommerce->cart;

    if ($cart) {
      foreach($cart->get_cart() as $cart_item_key => $cart_item) {
        // Cart items are iterated in the order they were added
        // So the "latest" strategy picks the last elt of the array
        if ($strategy === self::CART_REMINDER_STRATEGY_LATEST) {
          $selected_cart_item = $cart_item;
          continue;
        }

        $product = $cart_item['data'];
        $quantity = $cart_item['quantity'];
        $cart_item_total = $product->get_price() * $quantity;

        // We must select at least a product
        if ($selected_cart_item === null) {
          $selected_cart_item = $cart_item;
          $selected_cart_item_total = $cart_item_total;
          continue;
        }
        switch($strategy) {
          case self::CART_REMINDER_STRATEGY_MOST_EXPENSIVE:
            if ($cart_item_total > $selected_cart_item_total) {
              $selected_cart_item = $cart_item;
              $selected_cart_item_total = $cart_item_total;
            }
            break;
          case self::CART_REMINDER_STRATEGY_LEAST_EXPENSIVE:
            if ($cart_item_total < $selected_cart_item_total) {
              $selected_cart_item = $cart_item;
              $selected_cart_item_total = $cart_item_total;
            }
            break;
        }
      }
    }

    $destination = WonderPushSettings::getSettings()->getCartReminderDestination();
    if (!$destination) $destination = WonderPushWooCommerce::CART_REMINDER_DESTINATION_HOMEPAGE;

    $message = WonderPushSettings::getSettings()->getCartReminderMessage();
    if (!$message) $message = null;

    $url = get_bloginfo('wpurl');
    switch ($destination) {
      case WonderPushWooCommerce::CART_REMINDER_DESTINATION_CHECKOUT:
        $url = wc_get_checkout_url();
        break;
      case WonderPushWooCommerce::CART_REMINDER_DESTINATION_CART:
        $url = wc_get_cart_url();
        break;
    }
    $product = $selected_cart_item ? $selected_cart_item['data'] : null;
    $pictureUrl = null;
    if ($product && !WonderPushSettings::getSettings()->getDisableCartReminderImage()) {
      if ( $product->get_image_id() ) {
        $pictureUrl = wp_get_attachment_url($product->get_image_id());
      } elseif ( $product->get_parent_id() ) {
        $parent_product = wc_get_product( $product->get_parent_id() );
        if ( $parent_product && $parent_product->get_image_id() ) {
          $pictureUrl = wp_get_attachment_url($parent_product->get_image_id());
        }
      }
    }
    try {
      WonderPushUtils::put_current_installation_properties($access_token, (object)array(
        'string_cartReminderProductName' => $product ? $product->get_name() : null,
        'string_cartReminderUrl' => $url,
        'string_cartReminderMessage' => $message,
        'string_cartReminderPictureUrl' => $pictureUrl ? $pictureUrl : null,
      ));
    } catch (Exception $e) {
      error_log('Could not set cart reminder properties: ' . $e->getMessage());
    }
  }
}